%{
#include <strings.h>

#include "cfg_file.tab.h"
#include "cfg_file.h"

#include "vtun.h"

/* static buffer for strings */
char str_buf[255];

int find_keyword(char *str);

#define ECHO {;}
#define YY_DECL int yylex(void)

int yyerror(char *str);
%}

number		[0-9]+
dnumber		{number}:{number}
word		[A-Za-z][A-Za-z0-9\-\_+=\!\$\#\%\&\*\^\@\\]*
keyword		[a-z_-]+	
comment		\#.*\n
ipaddr		{number}\.{number}\.{number}\.{number}
nmaddr		{word}(\.|{word})*
path		(\/{word})+
string		\".*\"

%x OPT

%%
{comment}	{ lineno++; } /* skip comments */

{keyword}	{
		  int kw = find_keyword(yytext);
		  yylval.str = yytext;

		  if( kw ){	
		     /* K_DEFAULT & K_OPTIONS is _not_ host options
		      * don't begin OPT stage for them */
		     if(kw != K_DEFAULT && kw != K_OPTIONS) BEGIN(OPT);
		     return kw;
		  }
		  /* Keyword not found, means host definition. */
		  return K_HOST;
		}

<OPT>"yes"	{
		  yylval.num = 1;
		  return BOOL;
		}		 

<OPT>"no"	{
		  yylval.num = 0;
		  return BOOL;
		}		  

<OPT>"tty"	{
		  yylval.num = VTUN_TTY;
		  return NUM;
		}

<OPT>"pipe"	{
		  yylval.num = VTUN_PIPE;
		  return NUM;
		}

<OPT>"ether"	{
		  yylval.num = VTUN_ETHER;
		  return NUM;
		}		 

<OPT>{string}	{
		  if(yyleng > sizeof(str_buf)-1){
		     yyerror("string to long");
		     return 0;
		  }
		  strncpy(str_buf, yytext+1, yyleng-2);
		  str_buf[yyleng-2] = '\0';

		  yylval.str = str_buf;
					  
		  return STRING;
		}

<OPT>{word}	{
		  yylval.str = yytext;
		  return WORD;
		}

<OPT>{dnumber}	{
	          char *ptr = strchr(yytext,':') + 1;
		 
		  yylval.dnum.num1 = atoi(yytext);
		  yylval.dnum.num2 = atoi(ptr);
		  return DNUM;
		}

<OPT>{number}	{
		  yylval.num = atoi(yytext);
		  return NUM;
		}

<OPT>{ipaddr}	{
		  yylval.str = yytext;
		  return IPADDR;
		}

<OPT>{nmaddr}	{
		  yylval.str = yytext;
		  return NMADDR;
		}

<OPT>{path}	{
		  yylval.str = yytext;
		  return PATH;
		}

<OPT>\;		BEGIN(0);

\n		{ lineno++; } 

[ \t]		; /* skip whitespace */

.		return *yytext;

%%

int yywrap(void) 
{
    return 1;
}


int find_keyword(char *str)
{
    int i = -1;

    while( kword[++i].str )
	if( !strncmp(kword[i].str, str, 4) )
	   return kword[i].type;
    return 0;
}




