%{
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <syslog.h>

#include "vtun.h"

int lineno = 1;

struct host default_host;
struct host *parse_host;

char * search_host;

extern struct host *host_list;

int yyparse(void);
int yylex(void);	
int yyerror(char *s); 
%}

%union {
   char *str;
   int  num;
   struct { int num1; int num2; } dnum;
}

%token K_OPTIONS K_DEFAULT K_PORT K_PERSIST K_TIMEOUT K_PASSWD 
%token K_PROG K_ARGS K_SPEED
%token K_TYPE K_COMPRESS K_ENCRYPT  

%token <str> K_HOST 

%token <str> WORD IPADDR NMADDR STRING PATH
%token <num> NUM BOOL
%token <dnum> DNUM

%%
config: 
  | config statement    
  ;

statement: '\n'
  | K_OPTIONS   '{' options '}' 

  | K_DEFAULT   { 
		  parse_host = &default_host; 
                }        
    '{' host_options '}' 
  | K_HOST      { 
		  /* First check is it host we are looking for
		   * if not skip it */
		  if( search_host ){
		     if( strcmp(search_host, $1) )
			break; 	
		     search_host = (char*) -1;
		  }

		  if( !(parse_host = malloc(sizeof(struct host))) ){
		     yyerror("No memory for the host");
		     return -1;
		  }

		  /* Fill new host struct with default values,
		   * have to _dup_ strings to be able to reread config. */
	  	  memcpy(parse_host, &default_host, sizeof(struct host));
		  parse_host->host = strdup($1);
		  parse_host->passwd = NULL;
		  if(default_host.prog)
		     parse_host->prog = strdup(default_host.prog);
		  if(default_host.args)
		     parse_host->args = strdup(default_host.args);

		  /* Add host to the list */
		  parse_host->next = host_list;
		  host_list = parse_host;
		}    
    '{' host_options '}' 
		{
		  /* Return if searching host is found */		
	          if( search_host == (char*)-1 )
    		     return 0; 
		}
  ;

options:
    option
  | options option
  ;

/* Don't overide command line options */
option: '\n'
  | K_PORT NUM 		{ 
			   if(vtun.port == -1)
			      vtun.port = $2;
			}
  | K_PERSIST BOOL 	{ 
			   if(vtun.persist == -1) 
			      vtun.persist = $2; 	
			}
  | K_TIMEOUT NUM 	{  
			   if(vtun.timeout == -1)
			      vtun.timeout = $2; 	
			}
  ;

host_options:
    host_option
  | host_options host_option
  ;

/* Host options. Have to free strings first, because they 
 * could be strduped from default_host */
host_option: '\n'
  | K_PASSWD WORD 	{ 
			  free(parse_host->passwd);  
			  parse_host->passwd = strdup($2); 	
			}	
  | K_PROG PATH 	{   
			  free(parse_host->prog);  
			  parse_host->prog = strdup($2); 	
			}
  | K_ARGS STRING 	{   
			  free(parse_host->args);  
			  parse_host->args = strdup($2); 	
			}		
  | K_SPEED NUM 	{ 
			  if( $2 ){ 
			     parse_host->spd_in = parse_host->spd_out = $2;
			     parse_host->flags |= VTUN_SHAPE;
			  }
			}
  | K_SPEED DNUM 	{ 
			  if( yylval.dnum.num1 || yylval.dnum.num2 ){ 
			     parse_host->spd_out = yylval.dnum.num1;
		             parse_host->spd_in = yylval.dnum.num2; 	
			     parse_host->flags |= VTUN_SHAPE;
			  }
			}
  | K_COMPRESS BOOL 	{  
			  if($2) 
			    parse_host->flags |= VTUN_COMPRESS; 	
			}
  | K_ENCRYPT BOOL 	{  
			  if($2) 
			    parse_host->flags |= VTUN_ENCRYPT; 	
			}
  | K_TYPE NUM 		{  
			  parse_host->flags &= ~VTUN_TYPE_MASK;
			  parse_host->flags |= $2;
			  break; 
			}	
  ;
%%

int yyerror(char *s) 
{
   syslog(LOG_ERR, "Error \'%s\', line %d\n", s, lineno);
   return 0;
}

/* Find host:passwd in the host list */ 
struct host * find_host(char *host,char *pwd)
{
   struct host *host_ptr = host_list;

   while( host_ptr ){
     if( !strcmp(host_ptr->host,host) && !strcmp(host_ptr->passwd,pwd) )
	return host_ptr;
     host_ptr = host_ptr->next;
   } 
   return NULL;
}

void free_host_list(void)
{
   struct host *host_ptr, *next = host_list;

   free(default_host.passwd);   
   free(default_host.prog);   
   free(default_host.args);   
   memset(&default_host, 0, sizeof(struct host));

   while( (host_ptr = next) ){
     free(host_ptr->host);   
     free(host_ptr->passwd);   
     free(host_ptr->prog);   
     free(host_ptr->args);   
     next = host_ptr->next;
     free(host_ptr);
   } 

   host_list = NULL;
}

/* 
 * Read config file. 
 * If host arg is not NULL, only that host will be read.
 */ 

struct host * read_config(char *file, char *host) 
{
   static int list_filled = 0;
   extern FILE *yyin;

   if( list_filled )
      free_host_list(); 

   list_filled = 1;

   /* Initialize default options */
   default_host.flags = VTUN_TTY; 

   if( !(yyin = fopen(file,"r")) ){
      syslog(LOG_ERR,"Can not open %s", file);
      return NULL;      
   }

   search_host = host;

   yyparse();

   fclose(yyin);

   return host_list;
}
