/*  
    VTun - Virtual Tunnel over TCP/IP network.

    Copyright (C) 1998,1999  Maxim Krasnyansky <max_mk@yahoo.com>

    VTun has been derived from VPPP package by Maxim Krasnyansky. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 */

/*
 * Version: 1.0 03/31/1999 Maxim Krasnyansky <max_mk@yahoo.com>
 */ 
#include <unistd.h>
#include <stdlib.h>
#include <syslog.h>
#include <zlib.h>

#include "vtun.h"
#include "linkfd.h"
#include "lib.h"

/* Compressor module */

static z_stream zi, zd; 
static char *zbuf;
static int zbuf_size = LINKFD_BUF_SIZE + 200;

/* 
 * Initialize compressor/decompressor.
 * Allocate the buffer.
 */  

int alloc_zcomp(struct host *host)
{
     zd.zalloc = (alloc_func)0;
     zd.zfree  = (free_func)0;
     zd.opaque = (voidpf)0;
     zi.zalloc = (alloc_func)0;
     zi.zfree  = (free_func)0;
     zi.opaque = (voidpf)0;
    
     if( deflateInit(&zd, Z_BEST_SPEED ) != Z_OK ){
	syslog(LOG_ERR,"Can't initialize compressor");
	return 1;
     }	
     if( inflateInit(&zi) != Z_OK ){
	syslog(LOG_ERR,"Can't initialize decompressor");
	return 1;
     }	
     if( !(zbuf = malloc(zbuf_size)) ){
	syslog(LOG_ERR,"Can't allocate buffer");
	return 1;
     }	
     return 0;
}

/* 
 * Deinitialize compressor/decompressor.
 * Free the buffer.
 */  

int free_zcomp()
{
     deflateEnd(&zd);
     inflateEnd(&zi);
     free(zbuf);
     return 0;
}

int shr_zbuf(z_stream *zs, int len)
{
     if( !(zbuf = realloc(zbuf,zbuf_size+len)) )
         return -1;
     zs->next_out = zbuf + zbuf_size;
     zs->avail_out = len;
     zbuf_size += len;     

     return 0;
}

/* 
 * This functions _MUST_ consume all incoming bytes in one pass,
 * That's why we shrik buffer dinamicly.
 * Practic shows that buffer will not grow larger that 16K.
 */  
int zcomp(int len, char *in, char **out)
{ 
     int oavail, olen = 0;    
     int err;
 
     zd.next_in = in;
     zd.avail_in = len;
     zd.next_out = zbuf;
     zd.avail_out = zbuf_size;
    
     while(1){
        oavail = zd.avail_out;
        if( (err=deflate(&zd, Z_SYNC_FLUSH)) != Z_OK ){
           syslog(LOG_ERR,"Deflate error %d",err);
           return -1;
        }
        olen += oavail - zd.avail_out;
        if(!zd.avail_in)
	   break;

        if( shr_zbuf(&zd,100) ) 
           return -1;
     }
     *out = zbuf;
     return olen;
}

int zdecomp(int len, char *in, char **out)
{
     int oavail = 0, olen = 0;     
     int err;

     zi.next_in = in;
     zi.avail_in = len;
     zi.next_out = zbuf;
     zi.avail_out = zbuf_size;

     while(1){
        oavail = zi.avail_out;
        if( (err=inflate(&zi, Z_SYNC_FLUSH)) != Z_OK ){
           syslog(LOG_ERR,"Inflate error %d",err);
           return -1;
        }
        olen += oavail - zi.avail_out;
        if(!zi.avail_in)
	   break;
        if( shr_zbuf(&zi,100) )
           return -1;
     }
     *out = zbuf;
     return olen;
}

struct lfd_mod lfd_zcomp = {
     "Compressor",
     alloc_zcomp,
     zcomp,
     zdecomp,
     free_zcomp,
     NULL,NULL
};
