/*  
    VTun - Virtual Tunnel over TCP/IP network.

    Copyright (C) 1998,1999  Maxim Krasnyansky <max_mk@yahoo.com>

    VTun has been derived from VPPP package by Maxim Krasnyansky. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 */

/*
 * Version: 1.0 03/31/1999 Maxim Krasnyansky <max_mk@yahoo.com>
 */ 
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <syslog.h>
#include <strings.h>
#include <errno.h>

#include <sys/time.h>
#include <sys/resource.h>
#include <sched.h>

#include "vtun.h"
#include "linkfd.h"
#include "lib.h"

extern int errno;

/* Module functions */

struct lfd_mod *lfd_mod_head = NULL, *lfd_mod_tail = NULL;

/* Add module to the end of modules list */
int lfd_add_mod(struct lfd_mod *mod)
{
     if( !lfd_mod_head ){
        lfd_mod_head = lfd_mod_tail = mod;
	mod->next = mod->prev = NULL;
     } else {
        lfd_mod_tail->next = mod;
        mod->prev = lfd_mod_tail;
        mod->next = NULL;
        lfd_mod_tail = mod;
     }
     return 0;
}

/*  Initialize and allocate each module */
int lfd_alloc_mod(struct host * host)
{
     struct lfd_mod *mod = lfd_mod_head;
     int err = 0;  

     while( mod ){
        if( mod->alloc )
	   if( (err = (mod->alloc)(host)) )
	      break;
	mod = mod->next;
     } 

     return err;
}

/* Free each module */
int lfd_free_mod(void)
{
     struct lfd_mod *mod = lfd_mod_head;
     int err = 0;  

     while( mod ){
        if( mod->free )
	   if( (err = (mod->free)()) )
	      break;
	mod = mod->next;
     } 

     return err;
}

/* Run modules list DOWN or UP */ 
#define LFD_DOWN 1
#define LFD_UP   2

int lfd_run_mod(int len, char *in, char **out, int direction)
{
     struct lfd_mod *mod;
     
     if( !lfd_mod_head ){
	*out = in;
	return len;
     }

     if( direction == LFD_DOWN ){
	mod = lfd_mod_head;	
        while( mod ){
	   if( (len = (mod->encode)(len,in,out)) == -1 )
	      break;
           in = *out;
           mod = mod->next;
	}
     } else {
	mod = lfd_mod_tail;
        while( mod ){
	   if( (len = (mod->decode)(len,in,out)) == -1 )
	      break;
           in = *out;
           mod = mod->prev;
	}
     }

     return len;
}

/* Shaper */
int lfd_err = 0;
int lfd_len = 0; 
int lfd_fd1 = 0;
int lfd_fd2 = 0;
     
char *lfd_buf;

void lfd_shaper_timer(int sig){
     static struct timeval tv={0,0};
     static fd_set fdset;
     char *out = lfd_buf;
     register int len;

     FD_ZERO(&fdset);
     FD_SET(lfd_fd2, &fdset);

     /* 
      * Check for available data, or we will sleep in the read call 
      */
     if( (len = select(lfd_fd2+1, &fdset, NULL, NULL, &tv)) == -1) {
	lfd_err = 1;
	return;
     }
  
     if( !len )
	return;

     if( (len=read(lfd_fd2, lfd_buf, lfd_len)) <= 0 )
 	lfd_err = 1;
     else {
        if( (len = lfd_run_mod(len,lfd_buf,&out,LFD_DOWN)) == -1 )
	   lfd_err = 1;	
	else
   	   if( write_n(lfd_fd1, out, len) < 0 )
	      lfd_err = 1;
     }
}

int lfd_shaper(int fd1, int fd2, int speed){
     struct itimerval itm;
     struct sigaction sa;
     char *buf, *out;

     if(speed < 64)
	  return 1;
     /*
      * Calculate how much we will copy each timer tick. 
      * It is very primitive shaping, but shaping results 
      * is prety accurate.
      */
 
     lfd_len=LINKFD_MAGIC * (speed / 64);

     if( !(lfd_buf=malloc(lfd_len)) )
         return 1; 

     if( !(buf=malloc(LINKFD_BUF_SIZE)) )
        return 1; 
     out = buf;

     lfd_fd1 = fd1;
     lfd_fd2 = fd2;
     lfd_err = 0;

     /* Set up the SIGALRM handler */
     memset(&sa,0,sizeof(sa));    
     sa.sa_handler=lfd_shaper_timer;
     sigemptyset(&sa.sa_mask);
     sigfillset(&sa.sa_mask);
     sigaction(SIGALRM,&sa,NULL);    

     /* Start the timer */
     memset(&itm,0,sizeof(itm));
     itm.it_interval.tv_usec = itm.it_value.tv_usec = LINKFD_FREQ;
     setitimer(ITIMER_REAL,&itm,NULL);
     
     while(!lfd_err){
        register int len;

	errno = 0;
	if( (len=read(fd1, buf, LINKFD_BUF_SIZE)) <= 0 )
	   if( errno == EINTR )
	      continue;
	   else
	      break; 
        if( (len = lfd_run_mod(len,buf,&out,LFD_UP)) == -1 )
	   break;	 
	if( write_n(fd2, out, len) < 0 )
   	   break; 
     }

     memset(&itm,0,sizeof(itm));
     setitimer(ITIMER_REAL,&itm,NULL);

     sa.sa_handler=SIG_DFL;
     sa.sa_flags = 0;
     sigaction(SIGALRM,&sa,NULL);    

     free(lfd_buf);
     free(buf);
	
     return 0;
}
/* End of Shaper */


/* Fast linker */

int lfd_linker(int fd1,int fd2){
     int maxfd;
     fd_set fdset;
     char *buf, *out;

     if( !(buf=malloc(LINKFD_BUF_SIZE)) )
        return 1; 
     out = buf;

     maxfd = (fd1 > fd2 ? fd1 : fd2) + 1;

     while(1){
        register int len;

        /* Wait for data */
        FD_ZERO(&fdset);
	FD_SET(fd1, &fdset);
	FD_SET(fd2, &fdset);
	if(select(maxfd, &fdset, NULL, NULL, NULL) < 0 )
	   break;

	if( FD_ISSET(fd1, &fdset) ){
	   if( (len=read(fd1, buf, LINKFD_BUF_SIZE)) <= 0 )
	      break;  
	   if( (len = lfd_run_mod(len,buf,&out,LFD_UP)) == -1 )
	      break;	 
	   if( write_n(fd2, out, len) < 0 )
	      break; 
	}
	if( FD_ISSET(fd2, &fdset) ){
	   if( (len=read(fd2, buf, LINKFD_BUF_SIZE)) <= 0 )
		break;
	   if( (len = lfd_run_mod(len,buf,&out,LFD_DOWN)) == -1 )
	      continue;	 
	   if( write_n(fd1, out, len) < 0 )
		break;
	}
     }
     free(buf);

     return 0;
}

/* Link remote and local file descriptors */ 

int linkfd(struct host *host)
{
     int old_prio;

     old_prio=getpriority(PRIO_PROCESS,0);
     setpriority(PRIO_PROCESS,0,LINKFD_PRIO);

     if(host->flags & VTUN_COMPRESS)
	lfd_add_mod(&lfd_zcomp);

     if(host->flags & VTUN_ENCRYPT)
	lfd_add_mod(&lfd_encrypt);

     if( lfd_alloc_mod(host) )
	return 1;

     if(host->flags & VTUN_SHAPE)
	lfd_shaper(host->rmt_fd, host->loc_fd, host->spd_out);
     else     
	lfd_linker(host->rmt_fd, host->loc_fd);

     lfd_free_mod();

     setpriority(PRIO_PROCESS,0,old_prio);

     return 0;
}
