/*  
    VTun - Virtual Tunnel over TCP/IP network.

    Copyright (C) 1998,1999  Maxim Krasnyansky <max_mk@yahoo.com>

    VTun has been derived from VPPP package by Maxim Krasnyansky. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 */

/*
 * Version: 1.0 03/31/1999 Maxim Krasnyansky <max_mk@yahoo.com>
 */ 
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <syslog.h>
#include <signal.h>

#include "vtun.h"
#include "lib.h"

struct options vtun = {
	0,		/* Server */
	1,		/* Daemon */
	-1, 	        /* Port   */ 
	-1,		/* Persist */
	-1,		/* Timeout */
	VTUN_CONFIG_FILE
};

struct host *host_list = NULL;

void reread_config(int sig);

void usage(void);

int main(int argc, char *argv[], char *env[]){
     extern int optind,opterr,optopt;
     extern char *optarg;
     char *svr = NULL, *hst = NULL;
     struct sigaction sa;
     struct host *host = NULL;
     int opt;

     while( (opt=getopt(argc,argv,"sf:P:t:np")) != EOF ){
	switch(opt){
	    case 's':
		vtun.server = 1;
		break;
	    case 'P':
		vtun.port = atoi(optarg);
		break;
	    case 'f':
		vtun.cfg_file = strdup(optarg);
		break;
	    case 'n':
		vtun.daemon = 0;
		break;
	    case 'p':
		vtun.persist = 1;
		break;
	    case 't':
	        vtun.timeout = atoi(optarg);	
	        break;
	    default:
		usage();
	        exit(1);
	}
     }	

     openlog("vtund", LOG_PID, LOG_DAEMON);

     if(vtun.server){
	reread_config(0);
     } else {
	if( argc - optind < 2 ){
	   usage();
           exit(1);
	}
	hst = strdup(argv[optind++]);
	svr = strdup(argv[optind]);
	
        if( !(host = read_config(vtun.cfg_file, hst)) ){	
	   syslog(LOG_ERR,"Can't find host in %s",vtun.cfg_file);
	   exit(1);
        }
     } 
      	
     /* 
      * Now fill uninitialized fields of the options structure
      * with default values. 
      */ 

     if(vtun.port == -1)
	vtun.port = VTUN_PORT;
     if(vtun.persist == -1)
	vtun.persist = 0;		
     if(vtun.timeout == -1)
	vtun.timeout = VTUN_CONNECT_TIMEOUT;

     if(vtun.daemon){
        if( fork() )
	   exit(0);

	close(0);close(1);close(2);
	setsid();

	chdir("/");
     }

     if(vtun.server){
        memset(&sa,0,sizeof(sa));     
        sa.sa_handler=reread_config;
        sigaction(SIGHUP,&sa,NULL);

        init_title(argc,argv,env,"vtund[s]: ");
	server();
     } else {	
        init_title(argc,argv,env,"vtund[c]: ");
        client(host,svr);
     }

     closelog();
	
     return 0;
}

void reread_config(int sig)
{
     if( !read_config(vtun.cfg_file,NULL) ){
	syslog(LOG_ERR,"No hosts defined");
	exit(1);
     }
}

void usage(void)
{
     printf("Usage: \n");
     printf("  Server:\n");
     printf("\tvtund <-s> [-f file] [-P port]\n");
     printf("  Client:\n");
     printf("\tvtund [-f file] [-P port] [-p] [-t timeout] <host> <server adress>\n");
}


